/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: config.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "config.h"

#include "script/script.h"

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>

#include <qmessagebox.h>
#include <cstdlib>	// exit

namespace
{
	inline QTextStream& operator<<( QTextStream& out, bool b )
	{
		return out << static_cast<int>(b);
	}

	inline QTextStream& operator>>( QTextStream& in, bool& b )
	{
		int vb;
		in >> vb;
		b = vb == 1;
		return in;
	}

	inline QTextStream& operator<<( QTextStream& out, const QColor& c )
	{
		return out	<< c.red()		<< '\n'
					<< c.green()	<< '\n'
					<< c.blue()
					;
	}

	inline QTextStream& operator>>( QTextStream& in, QColor& c )
	{
		int r, g, b;
		in >> r >> g >> b;
		c.setRgb( r, g, b );
		return in;
	}

	inline QTextStream& operator<<( QTextStream& out, const Script& s )
	{
		return out	<< s.on()		<< '\n'
					<< s.name()		<< '\n'
					<< s.params()
					;
	}

	inline QTextStream& operator>>( QTextStream& in, Script& s )
	{
		QString n, p;
		bool on;
		in >> on >> n >> p;
		s.setName( n );
		s.setParams( p );
		s.setOn( on );
		return in;
	}

	const int FormatVersion	= 1;	//	version 1 du fichier de config
};

Config& Config::instance()
{
	static Config uniqConfig;
	return uniqConfig;
}

int Config::save()
{
	QFile f(file_);
	if ( f.open( IO_WriteOnly ) )
	{
		QTextStream stream( &f );
		stream	<< FormatVersion		<< '\n'

				//	--- Editeur piste ---
				<< pathTracks_			<< '\n'
				<< trackEditorSave_		<< '\n'
				<< trackEditorX_		<< '\n'
				<< trackEditorY_		<< '\n'
				<< paletteColorX_		<< '\n' 
				<< paletteColorY_		<< '\n'

				//	--- Piste de course ---
				<< pathDrivers_			<< '\n'
				<< trackSimulatorSave_	<< '\n'
				<< trackSimulatorX_		<< '\n' 
				<< trackSimulatorY_		<< '\n'
				<< windowResultsX_		<< '\n'
				<< windowResultsY_		<< '\n'
				<< windowLogX_			<< '\n'
				<< windowLogY_			<< '\n'
				<< windowLogW_			<< '\n'
				<< windowLogH_			<< '\n'

				//	--- Animation ---
				<< driverColor_			<< '\n'
				<< crossColor_			<< '\n'
				<< traceColor_			<< '\n'
				<< crossSize_			<< '\n'
				<< timer_				<< '\n'

				<<	Script::instance()
				;

		f.close();
	}
	return f.status();
}

Config::Config()
:	 file_("f1.configuration")
	,modif_(false)
{
	const QString env = getenv( "PISTEF1" );
	workingPath_ =		env.isNull()
					?	QDir::homeDirPath() + "/PisteF1"
					:	env
					;

	if ( ! QFile::exists( workingPath_ ) )
	{
		QMessageBox::critical( 0, "INITIALISATION",
			"Le rpertoire " + workingPath_ + " n'existe pas.\n"
			"revoir l'installation ou dfinir la variable d'environnement PISTEF1."
			);
		std::exit( 3 );
	}

	if ( ! QDir::setCurrent( workingPath_ ) )
	{
		QMessageBox::critical( 0, "INITIALISATION",
			"Ne peut ce dplacer dans le rpertoire : " + workingPath_ );
		std::exit( 3 );
	}

	QFile f(file_);
	if ( f.open( IO_ReadOnly ) )
	{
		QTextStream stream( &f );
		int	formatVersion;
		stream	>> formatVersion;

		switch ( formatVersion )
		{
		case 1:
			stream
				//	--- Editeur piste ---
				>> pathTracks_
				>> trackEditorSave_
				>> trackEditorX_
				>> trackEditorY_
				>> paletteColorX_ 
				>> paletteColorY_

				//	--- Piste de course ---
				>> pathDrivers_
				>> trackSimulatorSave_
				>> trackSimulatorX_ 
				>> trackSimulatorY_
				>> windowResultsX_
				>> windowResultsY_
				>> windowLogX_
				>> windowLogY_
				>> windowLogW_
				>> windowLogH_

				//	--- Animation ---
				>> driverColor_
				>> crossColor_
				>> traceColor_
				>> crossSize_
				>> timer_

				>> Script::instance()
				;
			break;

		default:	//	Version inconnue
			initDefault();
		}
	}
	else
		initDefault();
}

void Config::initDefault()
{
	pathTracks_			= workingPath_ + "/circuits";
	pathDrivers_		= workingPath_ + "/pilotes";

	trackEditorSave_ = trackSimulatorSave_ = true;
	trackEditorX_		= 123;
	trackEditorY_		= 0;
	trackSimulatorX_	= 0;
	trackSimulatorY_	= 98;
	paletteColorX_		= paletteColorY_ = 0;
	windowResultsX_ 	= windowResultsY_ = windowLogX_ = windowLogY_ = 0;
	windowLogW_			= 400;
	windowLogH_			= 300;

	driverColor_		= Qt::darkMagenta;
	crossColor_			= Qt::yellow;
	traceColor_			= Qt::magenta;
	crossSize_			= 3;
	timer_				= timerStep/2;
}

