/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: config.h
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#ifndef CONFIG_H
#define CONFIG_H

#include <qstring.h>
#include <qcolor.h>

class Config
{
public:
	enum { timerMax = 1000, timerStep = timerMax/4 };

	static Config& instance();

	QString	workingPath()						const;

	QString	pathTracks()						const;
	void	setPathTracks( const QString& pt );
	QString	pathDrivers()						const;
	void	setPathDrivers( const QString& pp );

	void	saveTrackEditorXY( bool save );
	bool	saveTrackEditorXY()					const;
	void	setTrackEditorXY( int x, int y );
	int		trackEditorX()						const;
	int		trackEditorY()						const;

	void	saveTrackSimulatorXY( bool save );
	bool	saveTrackSimulatorXY()				const;
	void	setTrackSimulatorXY( int x, int y );
	int		trackSimulatorX()					const;
	int		trackSimulatorY()					const;

	void	setWindowResultsXY( int x, int y );
	int		windowResultsX()					const;
	int		windowResultsY()					const;

	void	setWindowLog( int x, int y, int w, int h );
	int		windoLogX()							const;
	int		windoLogY()							const;
	int		windoLogW()							const;
	int		windoLogH()							const;

	int		paletteColorX()						const;
	int		paletteColorY()						const;
	void	setPaletteColorXY( int x, int y );

	QColor	driverColor()						const;
	QColor	crossColor()						const;
	QColor	traceColor()						const;
	int		crossSize()							const;
	int		timer()								const;

	void	driverColor( const QColor& );
	void	crossColor( const QColor& );
	void	traceColor( const QColor& );
	void	crossSize( int size );
	void	setTimer( int ms );

	int		save();	// renvoie IO_Ok & CO

	~Config() { save(); }

private:
	Config();
	void initDefault();
	Config( const Config& ) {}
	void operator=( const Config& ) {}

private:
	const	QString	file_;
	bool			modif_;

	QString	workingPath_;

	QString	pathTracks_;
	QString pathDrivers_;

	bool	trackEditorSave_;
	int		trackEditorX_;
	int		trackEditorY_;

	bool	trackSimulatorSave_;
	int		trackSimulatorX_;
	int		trackSimulatorY_;
	int		windowResultsX_;
	int		windowResultsY_;
	int		windowLogX_;
	int		windowLogY_;
	int		windowLogW_;
	int		windowLogH_;

	int		paletteColorX_;
	int		paletteColorY_;

	QColor	driverColor_;
	QColor	crossColor_;
	QColor	traceColor_;
	int		crossSize_;
	int		timer_;
};

inline QString	Config::workingPath() const
{
	return workingPath_;
}

inline QString	Config::pathTracks() const 
{
	return pathTracks_; 
}

inline void	Config::setPathTracks( const QString& pt ) 
{
	pathTracks_ = pt; 
	modif_ = true;
}

inline QString	Config::pathDrivers() const
{
	return pathDrivers_; 
}

inline void	Config::setPathDrivers( const QString& pp )
{
	pathDrivers_ = pp;
	modif_ = true;
}

inline void Config::saveTrackEditorXY( bool save )
{
	trackEditorSave_ = save;
	modif_ = true;
}

inline bool	Config::saveTrackEditorXY() const
{
	return trackEditorSave_; 
}

inline void	Config::setTrackEditorXY( int x, int y )
{
	if ( trackEditorSave_ && (trackEditorX_ != x || trackEditorY_ != y) )
	{
		trackEditorX_ = x;
		trackEditorY_ = y;
		modif_ = true;
	}
}

inline int	Config::trackEditorX() const	{ return trackEditorX_; }
inline int	Config::trackEditorY() const	{ return trackEditorY_; }

inline void	Config::saveTrackSimulatorXY( bool save )
{
	trackSimulatorSave_ = save;
	modif_ = true;
}

inline bool	Config::saveTrackSimulatorXY() const { return trackSimulatorSave_; }

inline void	Config::setTrackSimulatorXY( int x, int y )
{
	if ( trackSimulatorSave_ && (trackSimulatorX_ != x || trackSimulatorY_ != y) )
	{
		trackSimulatorX_ = x;
		trackSimulatorY_ = y;
		modif_ = true;
	}
}

inline int	Config::trackSimulatorX() const	{ return trackSimulatorX_; }
inline int	Config::trackSimulatorY() const	{ return trackSimulatorY_; }

inline	void Config::setWindowResultsXY( int x, int y )
{
	if ( windowResultsX_ != x || windowResultsY_ != y )
	{
		windowResultsX_ = x;
		windowResultsY_ = y;
		modif_ = true;
	}
}

inline	int	 Config::windowResultsX() const { return windowResultsX_; }
inline	int	 Config::windowResultsY() const { return windowResultsY_; }

inline void	Config::setWindowLog( int x, int y, int w, int h )
{
	if ( windowLogX_ != x || windowLogY_ != y
									|| windowLogW_ != w || windowLogH_ != h )
	{
		windowLogX_ = x;
		windowLogY_ = y;
		windowLogW_ = w;
		windowLogH_ = h;
		modif_ = true;
	}
}

inline int	Config::windoLogX() const	{ return windowLogX_; }
inline int	Config::windoLogY()	const	{ return windowLogY_; }
inline int	Config::windoLogW() const	{ return windowLogW_; }
inline int	Config::windoLogH()	const	{ return windowLogH_; }

inline int	Config::paletteColorX() const { return paletteColorX_; }
inline int	Config::paletteColorY() const { return paletteColorY_; }
inline void	Config::setPaletteColorXY( int x, int y )
{
	if ( trackEditorSave_ && (paletteColorX_ != x || paletteColorY_ != y) )
	{
		paletteColorX_ = x;
		paletteColorY_ = y;
		modif_ = true;
	}
}

inline QColor	Config::driverColor()	const { return driverColor_; }
inline QColor	Config::crossColor()	const { return crossColor_; }
inline QColor	Config::traceColor()	const { return traceColor_; }
inline int		Config::crossSize()		const { return crossSize_; }
inline int		Config::timer()			const { return timer_; }

inline void	Config::driverColor( const QColor& c )
{
	driverColor_ = c;
	modif_ = true;
}

inline void	Config::crossColor(const QColor& c )
{
	crossColor_ = c;
	modif_ = true;
}

inline void	Config::traceColor(const QColor& c )
{
	traceColor_ = c;
	modif_ = true;
}

inline void	Config::crossSize( int size )
{
	if ( size != crossSize_ )
	{
		crossSize_ = size;
		modif_ = true;
	}
}

inline void Config::setTimer( int timer )
{
	if ( timer != timer_ )
	{
		timer_ = timer;
		modif_ = true;
	}
}

#endif	// CONFIG_H
	
