/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: dialogconfig.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "dialogconfig.h"

#include <qlayout.h>

#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qmessagebox.h>
#include <qapplication.h>	// desktop size
#include <qfile.h>
#include <qfiledialog.h>
#include <qtextstream.h>
#include <qtooltip.h>

#include "config.h"
#include "divers/ioerrors.h"
#include "wdivers/editselectfilewidget.h"
#include "wdivers/dialogpalettecolor.h"

namespace
{
	class DialogPaletteColor2 : public DialogPaletteColor
	{
	public:
		DialogPaletteColor2( QWidget* parent, const char* name=0 )
		:	 DialogPaletteColor( parent, name, true )
		{
			QPushButton* btnClose = new QPushButton( "Fermer", this );
			connect( btnClose, SIGNAL(clicked()), SLOT(accept()) );
			gridLayout_->addWidget( btnClose, 1, 0 );
		}
	};
};	// end namespace

class ColorButton : public QPushButton
{
Q_OBJECT
public:
	ColorButton( const QColor& pcolor, const QString& title, QWidget* parent )
	:	 QPushButton( title, parent )
		,color_(pcolor)
		,modif_(false)
	{
		connect( this, SIGNAL(clicked()), SLOT(selectColor()) );
		setColor();
	}

	bool  colorChanged() const { return modif_; }
	const QColor& color() const { return color_; }

private slots:
	void activeColor( const QColor& c )
	{
		if ( color_ != c )
		{
			color_ = c;
			modif_ = true;
		}
	}

	void selectColor()
	{
		DialogPaletteColor2 dpc( this );
		dpc.setColor( color_ );
		connect( &dpc, SIGNAL(colorChanged(const QColor&)),
										SLOT(activeColor(const QColor&)) );
		dpc.move( mapToParent( QPoint( x() + width(), y() ) ) );
		dpc.exec();
		if ( modif_ )
			setColor();
	}

private:
	void setColor()
	{
		QPalette p = palette();
		p.setColor( QPalette::Active, QColorGroup::ButtonText, color_ );
		p.setColor( QPalette::Inactive, QColorGroup::ButtonText, color_ );
		setPalette( p );
	}

private:
	QColor	color_;
	bool	modif_;
};

class WindowCoords : public QGroupBox
{
Q_OBJECT
public:
	WindowCoords( const QString& title, QWidget* parent )
	:	 QGroupBox( parent )
	{
		QGridLayout* gridLayout = new QGridLayout( this );
		gridLayout->setAlignment( Qt::AlignTop );
		gridLayout->setSpacing( 6 );
		gridLayout->setMargin( 11 );

		{	// spacer vertical
			QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
		    gridLayout->addItem( spacer, 0, 0 );
		}

		gridLayout->addWidget( new QLabel( title, this ), 1, 0 );

		pathWidget_= new EditSelectFileWidget( EditSelectFileWidget::Path, this );
		gridLayout->addWidget( pathWidget_, 2, 0 );

		{	// spacer vertical
			QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
		    gridLayout->addItem( spacer, 3, 0 );
		}

		{
			QFrame* hLineFrame = new QFrame( this );
			hLineFrame->setFrameStyle( QFrame::HLine | QFrame::Sunken );
			gridLayout->addWidget( hLineFrame, 4, 0 );
		}

		checkMemCoord_ = 
			new QCheckBox( "&Mmoriser les coordonnes  la fermeture", this );
		gridLayout->addWidget( checkMemCoord_, 5, 0 );

		QHBoxLayout* hLayoutXY = new QHBoxLayout;
		{
			const int	desktopWidth	= QApplication::desktop()->width();
			const int	desktopHeight	= QApplication::desktop()->height();

			QLabel* labelX = new QLabel( "&x = ", this );
			hLayoutXY->addWidget( labelX );
			spinBoxX_ = new QSpinBox( 0, desktopWidth, 100, this );
			labelX->setBuddy( spinBoxX_ );
			hLayoutXY->addWidget( spinBoxX_, 1 );

			hLayoutXY->addItem( new QSpacerItem( 100, 20 ) );

			QLabel* labelY = new QLabel( "&y = ", this );
			hLayoutXY->addWidget( labelY );
			spinBoxY_ = new QSpinBox( 0, desktopHeight, 50, this );
			labelY->setBuddy( spinBoxY_ );
			hLayoutXY->addWidget( spinBoxY_, 1 );
		}
		gridLayout->addLayout( hLayoutXY, 6, 0 );

		{	// spacer vertical
			QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
		    gridLayout->addItem( spacer, 7, 0 );
		}

		connect( checkMemCoord_, SIGNAL(toggled(bool)),
										this, SLOT(checkMemToggled(bool)) );
	}

	void setPath( const QString& path )	{ pathWidget_->setFile( path ); }
	QString path() const				{ return pathWidget_->file(); }

	void setMemCoord( bool on )	{ checkMemCoord_->setChecked( on ); }
	void setX( int x )			{ spinBoxX_->setValue( x ); }
	void setY( int y )			{ spinBoxY_->setValue( y ); }

	bool memCoord() const { return checkMemCoord_->isChecked(); }
	int  getX()		const { return spinBoxX_->text().toInt(); }
	int  getY()		const { return spinBoxY_->text().toInt(); }

public slots:
	void checkMemToggled( bool toggle )
	{
		spinBoxX_->setDisabled( toggle );
		spinBoxY_->setDisabled( toggle );
	}					

private:
	EditSelectFileWidget*	pathWidget_;
	QCheckBox*				checkMemCoord_;
	QSpinBox*				spinBoxX_;
	QSpinBox*				spinBoxY_;
};

DialogConfig::DialogConfig( QWidget* parent )
:	 QTabDialog( parent, "DialogConfig", true )
	,config_(Config::instance())
{
	setCaption( "Configuration" );

	//-----------------------------------------------------------------
	windowEditor_ = new WindowCoords( "Rpertoire des pistes (.cir)", this );
	if ( ! config_.pathTracks().isEmpty() )
		windowEditor_->setPath( config_.pathTracks() );
	windowEditor_->setMemCoord( config_.saveTrackEditorXY() );
	windowEditor_->setX( config_.trackEditorX() );
	windowEditor_->setY( config_.trackEditorY() );

	addTab( windowEditor_, "diteur piste" );
	//-----------------------------------------------------------------

	//-----------------------------------------------------------------
	windowSimulator_ = new WindowCoords( "Rpertoire des logs (.f1)", this );
	if ( ! config_.pathDrivers().isEmpty() )
		windowSimulator_->setPath( config_.pathDrivers() );
	windowSimulator_->setMemCoord( config_.saveTrackSimulatorXY() );
	windowSimulator_->setX( config_.trackSimulatorX() );
	windowSimulator_->setY( config_.trackSimulatorY() );

	addTab( windowSimulator_, "Simulateur" );
	//-----------------------------------------------------------------

	//-----------------------------------------------------------------
	tabCollision();
	//-----------------------------------------------------------------

	setCancelButton( "Annuler" );
	setApplyButton( "Appliquer" );
	connect( this, SIGNAL(applyButtonPressed()), this, SLOT(save()) );
	setOkButton( "Valider" );
}


void DialogConfig::tabCollision()
{
	QWidget* w = new QWidget( this );
	QGridLayout* gridLayout = new QGridLayout( w );
	gridLayout->setAlignment( Qt::AlignTop );
	gridLayout->setSpacing( 6 );
	gridLayout->setMargin( 11 );

	driverBtn_ = new ColorButton( config_.driverColor(),
													"Couleur pilote", w );
	gridLayout->addWidget( driverBtn_, 0, 0 );

	crossBtn_ = new ColorButton( config_.crossColor(),
												"Couleur croix colision", w );
	gridLayout->addWidget( crossBtn_, 1, 0 );

	traceBtn_ = new ColorButton( config_.traceColor(),
												"Couleur trace", w );
	gridLayout->addWidget( traceBtn_, 2, 0 );

    spinBoxCrossSize_ = new QSpinBox ( 3, 9, 1, w );
	spinBoxCrossSize_->setPrefix( "Taille croix colision: " );
	spinBoxCrossSize_->setValue( config_.crossSize() );
	spinBoxCrossSize_->setSuffix( " pixels" );
	gridLayout->addWidget( spinBoxCrossSize_, 3, 0 );

	{	// spacer vertical
		QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	    gridLayout->addItem( spacer, 4, 0 );
	}

	QHBoxLayout* hblayout = new QHBoxLayout;
	QLabel* labelTimer = new QLabel( "Vitesse animation", w );
	hblayout->addWidget( labelTimer );
	sliderTimerInterval_ =
			new QSlider( 0, Config::timerMax, Config::timerStep,
									config_.timer(), QSlider::Horizontal, w );
	sliderTimerInterval_->setTickmarks( QSlider::Both );
	hblayout->addWidget( sliderTimerInterval_, 1 );
	QLCDNumber* lcdTimer = new QLCDNumber( w );
	connect(	sliderTimerInterval_,	SIGNAL(valueChanged(int)),
				lcdTimer,				SLOT(display(int)) );
	lcdTimer->display( config_.timer() );
	hblayout->addWidget( lcdTimer );
	QToolTip::add( labelTimer, "Temps en ms: 1000 = vitesse relle" );
	QToolTip::add( sliderTimerInterval_, "Temps en ms: 1000 = vitesse relle" );
	QToolTip::add( lcdTimer, "Temps en ms: 1000 = vitesse relle" );

	gridLayout->addLayout( hblayout, 6, 0 );


	addTab( w, "Simulation/Debug" );
}

bool DialogConfig::save()
{
	config_.setPathTracks( windowEditor_->path() );
	config_.setPathDrivers( windowSimulator_->path() );

	config_.saveTrackEditorXY( windowEditor_->memCoord() );
	config_.setTrackEditorXY( windowEditor_->getX(), windowEditor_->getY() );

	config_.saveTrackSimulatorXY( windowSimulator_->memCoord() );
	config_.setTrackSimulatorXY( windowSimulator_->getX(), windowSimulator_->getY() );

	if ( driverBtn_->colorChanged() )
		config_.driverColor( driverBtn_->color() );
	if ( crossBtn_->colorChanged() )
		config_.crossColor( crossBtn_->color() );
	if ( traceBtn_->colorChanged() )
		config_.traceColor( traceBtn_->color() );

	QString bidon, b = spinBoxCrossSize_->text();
	int size;
	QTextIStream( &b ) >> bidon >> bidon >> bidon >> size;
	config_.crossSize( size );

	config_.setTimer( sliderTimerInterval_->value() );

	const int status = config_.save();
	if ( status != IO_Ok )
	{
		QMessageBox::critical( 0, "ERREUR FATAL",
				"Le fichier de configuration n'a pu tre sauvegard:\n\n"
				+ getIoError(status) );
		return false;
	}

	return true;
}

#include "dialogconfig.moc"

