/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: bmpascii.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "bmpascii.h"
#include <qmessagebox.h>
#include <qimage.h>
#include <qcolor.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qinputdialog.h> 

#include "widgettrack/dimensions.h"
#include "pixel/listshapepixel.h"
#include "ext.h"
#include "ioerrors.h"

namespace
{
	const QString	__TITLE__( "Conversion bmp -> cir");
};

namespace Convert
{
bool bmp2cir( const QString& bmpName, QString* cirName )
{
	const int lenExt = ext::track.length();
	*cirName = bmpName;
	cirName->replace( bmpName.length() - lenExt, lenExt, ext::track );
	
	QFile file( *cirName );
__RENAMED__:
	if ( file.exists() )
	{
		// on ne peut pas mettre 4 bouttons :(
		const int r = QMessageBox::warning( 0, __TITLE__,
					"Le fichier " + *cirName + " existe !",
					"craser", "Utiliser", "Renomer" );
		switch ( r  )
		{
		case 0:	// on crase.
			break;

		case 1:	// on utitise le fichier existant.
			return true;

		case 2:	// on renomme
			{
				bool ok;
				const QString ren = QInputDialog::getText ( "Renomer ?",
											*cirName + " en ", *cirName, &ok );
				if ( ok )
				{
					*cirName = ren;
					file.setName( ren );
					goto __RENAMED__;
				}
				else
					return false;
			}
			break;
		}
	}
			
	if ( file.open( IO_WriteOnly ) )
	{
		ListShapePixel& lsp 	= ListShapePixel::instance();
		const char defaultAscii = lsp[Vert].ascii();

		QImage bmpImage;
		if ( ! bmpImage.load( bmpName ) )
		{
			QMessageBox::critical( 0,  __TITLE__,
						"Impossible de charger " + bmpName );
			return false;
		}

		QTextStream stream( &file );

		int countErrors = 0;
		for ( int y = 0; y < Dim::maxY; ++y )
		{
			for ( int x = 0; x < Dim::maxX; ++x )
			{
				if ( ! lsp.find( QColor(bmpImage.pixel( x, y )) ) )
				{
					++countErrors;
					stream << defaultAscii;
				}
				else
					stream << lsp.ascii();
			}
			stream << '\n';
		}

		if ( countErrors != 0 )
			QMessageBox::warning( 0,  __TITLE__,
					"il y a eu " + QString::number(countErrors)
					+ " corrections !" );

		file.close();
	}

	if ( file.status() != IO_Ok )
	{
		QMessageBox::critical( 0,  __TITLE__,
			"Erreur lors de l'criture " +
					*cirName + ":\n" + getIoError( file.status() ) );
		return false;
	}
	return true;
}

};
