/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: mainf1.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "mainf1.h"
#include <qpopupmenu.h>
#include <qmessagebox.h>

#include "config/config.h"
#include "config/dialogconfig.h"
#include "trackeditor/trackeditor.h"
#include "tracksimulator/tracksimulator.h"
#include "icons/icons.h"
#include "wizard/wizardhelp.h"

MainF1::MainF1()
:	 QMenuBar( 0, "MainF1" )
	,upEditor_(false)
	,upSimulator_(false)
{
	setIconText( "F1 Racing" );
	setIcon( Icons::instance().mainF1() );

	QPopupMenu*	popupTrack = new QPopupMenu( this, "MainF1::popupTrack" );
	popupTrack->insertItem( "&Editer circuit", this, SLOT(trackEditor()) );
	popupTrack->insertItem( "&Simulateur", this, SLOT(trackSimulator()) );
	popupTrack->insertSeparator();
	popupTrack->insertItem( "C&onfiguration", this, SLOT(config()) );
	popupTrack->insertSeparator();
	popupTrack->insertItem( "&Quitter", this, SLOT(close()) );
	insertItem( "&Circuit", popupTrack );
	popupTrack->insertSeparator();
	popupTrack->insertItem( "Aide", this, SLOT(helpEditor()), CTRL+Key_F1 );
	QPopupMenu*	popupAbout = new QPopupMenu( this, "MainF1::popupAbout" );
	popupAbout->insertItem( "de F1", this, SLOT(aboutF1()) );
	popupAbout->insertItem( "de QT", this, SLOT(aboutQT()) );
	popupTrack->insertItem( "&A propos", popupAbout );

	setFixedSize( 110, 30 );
}

void MainF1::showEvent( QShowEvent* )
{
	if ( upEditor_ )
		trackEditor_->showNormal();

	if ( upSimulator_ )
		trackSimulator_->showNormal();
}

void MainF1::hideEvent( QHideEvent* )
{
	upEditor_		= trackEditor_		&& trackEditor_->isVisible();
	if ( upEditor_ )
		trackEditor_->showMinimized();

	upSimulator_	= trackSimulator_	&& trackSimulator_->isVisible();
	if ( upSimulator_ )
		trackSimulator_->showMinimized();
}

void MainF1::help( int idHelp )
{
	if ( ! wizardHelp_ )
	{
		wizardHelp_ = new WizardHelp(this);
		wizardHelp_->show();
	}
	else
		wizardHelp_->raise();
	wizardHelp_->gotoOn( static_cast<WizardHelp::Type>(idHelp) );

}

void MainF1::helpEditor()
{
	help( WizardHelp::Editor );
}

void MainF1::helpSimulator()
{
	help( WizardHelp::Simulator );
}

void MainF1::helpLog()
{
	help( WizardHelp::Log );
}

void MainF1::aboutF1()
{
	QMessageBox::information( this, "A propos de PisteF1",
    "<u>Version 1 :</u> 31/01/2003 dition et modification des pistes."
	"<br>"
	"<br><u>Version 2 :</u> <i>en cour</i> simulation d'une course."
	"<br>"
	"<br>voir <b>http://clx.anet.fr</b> pour les dtails du pilote logiciel et du concours."
	);
}

void MainF1::aboutQT()
{
	QMessageBox::aboutQt( this );
}

void MainF1::config()
{
	DialogConfig dc( this );
	dc.move( x(), height()*2 );
	if ( dc.exec() )
		dc.save();
}

void MainF1::trackEditor()
{
	if ( ! trackEditor_ )
	{
		const Config& conf = Config::instance();
		trackEditor_ = new TrackEditor( this, "MainF1::trackEditor_" );
		connect( trackEditor_, SIGNAL(help()), SLOT(helpEditor()) );
		if ( trackSimulator_ && ! trackSimulator_->trackName().isEmpty() )
			trackEditor_->load( trackSimulator_->trackName() );
		trackEditor_->move( conf.trackEditorX(), conf.trackEditorY() );
		trackEditor_->show();
		if ( trackEditor_->trackName().isEmpty() )
			trackEditor_->loadTrack();
	}
	else
	{
		trackEditor_->raise();
		trackEditor_->showNormal();
	}
}

void MainF1::trackSimulator()
{
	if ( ! trackSimulator_ )
	{
		const Config& conf = Config::instance();
		trackSimulator_ = new TrackSimulator( this, "MainF1::trackSimulator_" );
		connect( trackSimulator_, SIGNAL(help()), SLOT(helpSimulator()) );
		connect( trackSimulator_, SIGNAL(helpLog()), SLOT(helpLog()) );
		trackSimulator_->move( conf.trackSimulatorX(), conf.trackSimulatorY() );
		trackSimulator_->show();
	}
	else
	{
		trackSimulator_->raise();
		trackSimulator_->showNormal();
	}
}

void MainF1::closeEvent( QCloseEvent* ce )
{
	if ( ! trackEditor_ || trackEditor_->testSaveDialog() )
		ce->accept();
	else
		ce->ignore();
}

