/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: listshapepixel.h
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#ifndef LISTSHAPEPIXEL_H
#define	LISTSHAPEPIXEL_H

//============================================================================
//	ListShapePixel:
//		Gestion de toutes les formes que peut avoir un pixel.
//
//============================================================================

#include <qvector.h>
#include "shapepixel.h"
#include <cassert>

enum Couleur
{
	 Gris
	,Noir
	,RougeFonce
	,VertFonce
	,JauneFonce
	,BleuFonce
	,MagentaFonce
	,CyanFonce
	,GrisClair
	,Rouge
	,Vert
	,Jaune
	,Bleu
	,Magenta
	,Cyan
	,Blanc
};

class ListShapePixel
{
public:
	enum { max = 16 };
	static ListShapePixel&	instance();

	//	Recherche si le caractre ou la couleur existent.
	bool					find( char ascii );
	bool					find( const QColor& color );

	//	Renvoie les attributs de la forme trouve par find.
	char					ascii()	const;
	const QColor&			color()	const;
	const QString&			name()	const;
	const ShapePixel&		shape() const;

	//	Renvoie la forme d'un pixel sur son index, l'index doit tre valide.
	const ShapePixel&		operator[]( int indexOf ) const;
	//	Renvoie l'index d'une couleur, la couleur doit tre valide.
	static int				id( const QColor& );

private:
	ListShapePixel();
	ListShapePixel( const ListShapePixel& ) {}
	void operator=( const ListShapePixel& ) {}

	static bool	search( char ascii );
	static bool	search( const QColor& color );

private:
	typedef QVector<ShapePixel> MapListShapePixel;

	static MapListShapePixel	shapeVector_;
	static ShapePixel*			current_;
};


inline bool ListShapePixel::find( const QColor& color )
{
	return	( current_->color() == color )
		?	true
		:	search( color )
		;
}

inline bool ListShapePixel::find( char ascii )
{
	return	( current_->ascii() == ascii )
		?	true
		:	search( ascii )
		;
}

inline char					ListShapePixel::ascii()	const
{
	return current_->ascii(); 
}

inline const QColor&		ListShapePixel::color()	const
{
	return current_->color();
}

inline const QString&		ListShapePixel::name()	const
{
	return current_->name();
}

inline const ShapePixel&	ListShapePixel::shape() const
{
	return *current_;
}

inline const ShapePixel& ListShapePixel::operator[]( int indexOf ) const
{
	assert( indexOf >=0 && indexOf < max );
	return *shapeVector_[indexOf];
}



#endif	//	LISTSHAPEPIXEL_H


