/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: dialogscript.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "dialogscript.h"

#include "divers/ext.h"
#include "wdivers/editselectfilewidget.h"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qframe.h>

#include "script.h"

DialogScript::DialogScript( Script* script, QWidget* parent,  const char* name )
:	 QDialog( parent, name, true )
	,script_(script)
{
	setCaption( "Script" );
	QGridLayout* dialogScriptGridLayout = new QGridLayout( this ); 
	dialogScriptGridLayout->setSpacing( 6 );
	dialogScriptGridLayout->setMargin( 11 );

	{
		QHBoxLayout* layoutScriptName = new QHBoxLayout; 
		layoutScriptName->setSpacing( 6 );
		layoutScriptName->setMargin( 0 );
		
		QLabel* labelScripName = new QLabel( "Nom", this );
		layoutScriptName->addWidget( labelScripName );

		scriptPath_ = new EditSelectFileWidget( EditSelectFileWidget::File, this );
		scriptPath_->setFilters( "Excutable (*)" );
		scriptPath_->setFile( script->name() );
		layoutScriptName->addWidget( scriptPath_ );
		
		dialogScriptGridLayout->addLayout( layoutScriptName, 0, 0 );
	}

	{
		QHBoxLayout* layoutParams = new QHBoxLayout; 
		layoutParams->setSpacing( 6 );
		layoutParams->setMargin( 0 );

		QLabel* labelParams = new QLabel( "Paramtres:", this );
		layoutParams->addWidget( labelParams );

		scriptParams_ = new EditSelectFileWidget( EditSelectFileWidget::File, this );
		scriptParams_->setFilters( "*."+ext::track );
		scriptParams_->setFile( script->params() );
		layoutParams->addWidget( scriptParams_ );

		dialogScriptGridLayout->addLayout( layoutParams, 1, 0 );
	}

	{
		QFrame* hLine = new QFrame( this );
		hLine->setFrameStyle( QFrame::HLine | QFrame::Sunken );

		dialogScriptGridLayout->addWidget( hLine, 2, 0 );
	}

	{
		QHBoxLayout* layoutCtrl = new QHBoxLayout; 
		layoutCtrl->setSpacing( 6 );
		layoutCtrl->setMargin( 0 );

		checkBoxActif_ = new QCheckBox( "&Activer", this );
		checkBoxActif_->setChecked( script->on() );
		layoutCtrl->addWidget( checkBoxActif_ );

		QPushButton* btnOk = new QPushButton( "Ok", this );
		layoutCtrl->addWidget( btnOk );

		QPushButton* btnCancel = new QPushButton( "Annuler", this );
		layoutCtrl->addWidget( btnCancel );

		dialogScriptGridLayout->addLayout( layoutCtrl, 3, 0 );

		connect( btnOk, SIGNAL(clicked()), this, SLOT(save()) );
		connect( btnCancel, SIGNAL(clicked()), this, SLOT(reject()) );
	}
}

void DialogScript::save()
{
	script_->setName( scriptPath_->file() );
	script_->setParams( scriptParams_->file() );
	script_->setOn( checkBoxActif_->isChecked() );
	emit accept();
}

