/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: dialogselectarea.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "dialogselectarea.h"

#include <qlayout.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlcdnumber.h>

#include <cmath>	// std::abs

#include "wdivers/widgetinputpixel.h"

DialogSelectArea::DialogSelectArea( QWidget* parent, const char* name )
:	 QDialog( parent, name )
{
	setCaption( "Slectionner une zone" );
	QGridLayout* gridLayout = new QGridLayout( this ); 
	gridLayout->setSpacing( 6 );
	gridLayout->setMargin( 11 );

	topPixel_		= new WidgetInputPixel( "Pixel suprieur", this );
	gridLayout->addWidget( topPixel_, 0, 0 );
	connect( topPixel_, SIGNAL(valueChanged(int)), SLOT(newArea(int)) );
	bottomPixel_	= new WidgetInputPixel( "Pixel infrieur", this );
	gridLayout->addWidget( bottomPixel_, 1, 0 );
	connect( bottomPixel_, SIGNAL(valueChanged(int)), SLOT(newArea(int)) );

	{
		QFrame* hLineFrame = new QFrame( this );
		hLineFrame->setFrameStyle( QFrame::HLine | QFrame::Sunken );
		gridLayout->addWidget( hLineFrame, 2, 0 );
	}

	{
		QHBoxLayout* layoutTotalPixel = new QHBoxLayout; 
		layoutTotalPixel->setSpacing( 6 );
		layoutTotalPixel->setMargin( 0 );

		layoutTotalPixel->addWidget( new QLabel( "Total pixels", this ) );
		layoutTotalPixel->addItem( new QSpacerItem( 20, 20 ) );

    	lcdNumberTotalPixels_ = new QLCDNumber( this );
		lcdNumberTotalPixels_->setSegmentStyle( QLCDNumber::Flat );
	    layoutTotalPixel->addWidget( lcdNumberTotalPixels_ );

    	gridLayout->addLayout( layoutTotalPixel, 3, 0 );
	}

	{
		QFrame* hLineFrame = new QFrame( this );
		hLineFrame->setFrameStyle( QFrame::HLine | QFrame::Sunken );
		gridLayout->addWidget( hLineFrame, 4, 0 );
	}

	{
		QHBoxLayout* layoutButtons = new QHBoxLayout;
		layoutButtons->setSpacing( 6 );
		layoutButtons->setMargin( 0 );

		layoutButtons->addItem( new QSpacerItem( 20, 20 ) );

		QPushButton* btnOk = new QPushButton( "Coloriser", this );
		layoutButtons->addWidget( btnOk );
		connect( btnOk, SIGNAL(clicked()), SLOT(colorize()) );

		QPushButton* btnCancel = new QPushButton( "Fermer", this );
		layoutButtons->addWidget( btnCancel );
		connect( btnCancel, SIGNAL(clicked()), SIGNAL(close()) );

		gridLayout->addLayout( layoutButtons, 5, 0 );
	}
}

void DialogSelectArea::polish()
{
	QDialog::polish();
	setFixedSize( sizeHint() );
}

void DialogSelectArea::initStarterArea()
{
	blockSignals( true );		// on vite l'envoie de 4 signaux
	topPixel_->setXY( 20, 2 );
	bottomPixel_->setXY( 59, 6 );
	blockSignals( false );
	newArea( 0 /*bidon*/ );
}

int	DialogSelectArea::topLeftPixel()	 const { return topPixel_->pixel();}
int	DialogSelectArea::bottomRightPixel() const { return bottomPixel_->pixel();}

void DialogSelectArea::newArea( int )
{
	const int w = std::abs(bottomPixel_->pixelX() - topPixel_->pixelX()) + 1;
	const int h = std::abs(bottomPixel_->pixelY() - topPixel_->pixelY()) + 1;
	lcdNumberTotalPixels_->display( w * h );
	emit areaChanged( topPixel_->pixel(), bottomPixel_->pixel() );
}

void DialogSelectArea::colorize()
{
	emit paintArea( topPixel_->pixel(), bottomPixel_->pixel() );
	emit close();
}

