/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: trackeditor.h
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#ifndef TRACKEDITOR_H
#define TRACKEDITOR_H

//============================================================================
//	TrackEditor:
//		Permet d'diter et modifier une piste.
//
//============================================================================

#include "trackmainwindow/trackmainwindow.h"
#include "trackpixmapmanager.h"

class DialogPaletteColor;
class DialogSelectArea;
class QPushButton;
class Pixel;
class TrackEditor : public TrackMainWindow
{
Q_OBJECT
public:
	TrackEditor( QWidget* parent, const char* name );

	override void	show();
	using			TrackMainWindow::trackName;
	using			TrackMainWindow::loadTrack;
	override void	load( const QString& fileTrack );
	using			TrackMainWindow::testSaveDialog;

signals:
	void help();

private slots:
	void preview(bool);
	void redraw();		// redessine toute la piste.
	void adjust();		// vrifie la validit de la piste.
	void isChanged();	// modidie le titre sur une modification de la piste.
	void undo();		
	void redo();		

	//	Actions sur le click d'un pixel.
	void mouseLeftPressPixel( QMouseEvent*, Pixel& );
	void mouseMouveOnPixel( Pixel& );
	void mouseLeftReleasePixel( QMouseEvent*, Pixel& );

	//	Connecter  dialogSelectArea_
	void dialogSelectAreaChanged( int np0, int np1 );
	void dialogSelectAreaColorize( int np0, int np1 );
	void dialogSelectAreaClosed();

	//	Connecter  dialogSelectArea_
	void activeColorChanged( const QColor& );

private:
	override bool	save();
	void			fillSelection();
	override void	closeEvent( QCloseEvent* );

private:
	QPushButton*		btnSave_;

	QPushButton*		btnUndo_;
	QPushButton*		btnRedo_;
	TrackPixmapManager	trackImageManager_;

	enum Action { no, drawing, drawingBrush };
	Action			action_;

	Pixel*			mouseLeftPressPixel_;	// Mmorise le pixel slectionn
											// quand le boutton gauche de la
											// sourie a t press.
	Pixel*			mouseLeftReleasePixel_;	// Mmorise le pixel slectionn
											// quand le boutton gauche de la
											// sourie a t relach.

	QString			statusSelection_;	// Message initialis et envoy par
										// mouseLeftPressPixel, puis complte et
										// envoy par mouseMouveOnPixel et 
										// mouseLeftReleasePixel.

	DialogSelectArea*	dialogSelectArea_;

	DialogPaletteColor*	dialogPaletteColor_;
	QColor				activeColor_;
};


#endif	//TRACKEDITOR_H


