/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: trackpixmapmanager.h
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#ifndef TRACKPIXMAPMANAGER_H
#define TRACKPIXMAPMANAGER_H

//============================================================================
//	Gestion des images avant/aprs modifications permettant de faire
//	les 'undo/redo'.
//============================================================================

#include "widgettrack/trackpixmap.h"
#include <qstack.h>

class TrackPixmapManager
{
public:
	TrackPixmapManager();

	uint				countBefore()	const;
	uint				countAfter()	const;

	//	Dtruit les images afters.
	void				pushBefore( TrackPixmap* before );
	
	const TrackPixmap&	topBefore()	const;
	const TrackPixmap&	topAfter()	const;

	void				popBefore();
	void				popAfter();

	//	Cre une nouvelle image before initialis avec la zone de l'image
	//	top de after.
	TrackPixmap*		makeBefore();
	//	Cre une nouvelle image after initialis avec la zone de l'image
	//	top de before.
	TrackPixmap*		makeAfter();

private:
	typedef QStack<TrackPixmap> StackPixmap;
	StackPixmap	beforePixmap_;
	StackPixmap	afterPixmap_;
};


inline TrackPixmapManager::TrackPixmapManager()
{
	beforePixmap_.setAutoDelete( true );
	afterPixmap_.setAutoDelete( true );
}

inline uint TrackPixmapManager::countBefore()	const
	{ return beforePixmap_.count(); }
inline uint TrackPixmapManager::countAfter()	const
	{ return afterPixmap_.count(); }

inline void TrackPixmapManager::pushBefore( TrackPixmap* before )
{
	beforePixmap_.push( before );
	afterPixmap_.clear();
}
	
inline const TrackPixmap& TrackPixmapManager::topBefore()	const 
	{ return *beforePixmap_.top(); }
inline const TrackPixmap& TrackPixmapManager::topAfter()	const
	{ return *afterPixmap_.top(); }

inline void		TrackPixmapManager::popBefore()	{ beforePixmap_.pop(); }
inline void		TrackPixmapManager::popAfter()	{ afterPixmap_.pop(); }

inline TrackPixmap* TrackPixmapManager::makeAfter()
{
	TrackPixmap* n = new TrackPixmap( beforePixmap_.top()->rect() );
	afterPixmap_.push( n );
	return n;
}

inline TrackPixmap* TrackPixmapManager::makeBefore()
{
	TrackPixmap* n = new TrackPixmap( afterPixmap_.current()->rect() );
	beforePixmap_.push( n );
	return n;
}

#endif	// TRACKPIXMAPMANAGER_H

