/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: trackmainwindow.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "trackmainwindow.h"
#include "widgettrack/widgettrack.h"
#include "divers/ext.h"
#include "divers/bmpascii.h"
#include "config/config.h"
#include "icons/icons.h"

#include <qlayout.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qaccel.h>

namespace
{
	const QString _nottitle_( "PAS DE CIRCUIT" );
};

TrackMainWindow::TrackMainWindow( Type config, QWidget* parent,
															const char* name )
:	 QMainWindow( parent, name, WType_TopLevel | WDestructiveClose )
	,config_(Config::instance())
	,statusBar_(*statusBar())
{
	setIconText( "F1 Racing" );
	setIcon( Icons::instance().simulator() );
	setCaption( _nottitle_ );

	QAccel* accel = new QAccel( this );
	accel->connectItem( accel->insertItem(CTRL+Key_F1), this, SIGNAL(help()) );

	centralWidget_	= new QWidget( this ); 
	setCentralWidget( centralWidget_ );

	QVBoxLayout*	verticalLayout	= new QVBoxLayout( centralWidget_ );

	topLayout_ = new QHBoxLayout( 4 );
	topLayout_->setMargin( 2 );
	verticalLayout->addLayout( topLayout_, 0 );

	widgetTrack_ = 
		new WidgetTrack( config == TrackSimulator ? Dim::gap*2 : 0
						, centralWidget_, "TrackMainWindow::widgetTrack_" );
	verticalLayout->addWidget( widgetTrack_, 1, 0 );

	connect( widgetTrack_, SIGNAL(message(const QString&)),
								&statusBar_, SLOT(message(const QString&)) );
	connect( widgetTrack_, SIGNAL(warning(bool,const QString&)),
									this, SLOT(warning(bool,const QString&)) );
}

void TrackMainWindow::polish()
{
	QMainWindow::polish();
	setFixedSize( sizeHint() );
}

void TrackMainWindow::addTopBar( QWidget* add )
{
	topLayout_->addWidget( add );
}

void TrackMainWindow::addTopBar( QLayout* add )
{
	topLayout_->addLayout( add );
}

void TrackMainWindow::addSeparatorTopBar()
{
	topLayout_->addItem( new QSpacerItem( 10, 10 ) );
}

void TrackMainWindow::load( const QString& fileTrack )
{
	if ( widgetTrack_->load( fileTrack ) )
	{
		trackName_ = fileTrack;
		setCaption( fileTrack );
	}
}

bool TrackMainWindow::testSaveDialog()
{
	if ( widgetTrack_->state() == WidgetTrack::Changed )
	{
		const int r =
			QMessageBox::warning( this, "SAUVER",
						QString( "La piste %1 est modifie, enregistrer ?")
						.arg(trackName_), "Oui", "Non", "Annuler" );
		switch( r )
		{
		case 0:	// Oui
			return trackName_.isEmpty() ? saveDialog() : save();

		case 1:	// Non
			return true;

		case 2:	// Annuler
		default:
			return false;
		}
	}

	return true;
}

bool TrackMainWindow::save()
{
	if ( widgetTrack_->save( trackName_ ) )
	{
		setCaption( trackName_ );
		return true;
	}

	return false;
}

bool TrackMainWindow::saveDialog()
{
	QString file =
		QFileDialog::getSaveFileName( config_.pathTracks(), "*."+ext::track);
	if ( ! file.isEmpty() )
	{
		if ( file.mid( file.length() - 3 ) != ext::track )
			file += '.' + ext::track;

		trackName_ = file;
		return save();
	}

	return false;
}

void TrackMainWindow::loadTrack()
{
	const QString filters = "Pistes (*."+ext::track + " *.bmp)";
	if ( testSaveDialog() )
	{
		const QString file =
			QFileDialog::getOpenFileName( config_.pathTracks(),
										filters, NULL, 0, "Charger Piste" );

		if ( ! file.isEmpty() )
		{
			if ( file.contains( ".bmp", false ) != 0 )
			{
				QString cirName;
				if ( Convert::bmp2cir( file, &cirName ) )
					load( cirName );
				else
					warning( true,	"Impossible de convertir " + file +
														" -> " + cirName );
			}
			else
				load( file );
		}
	}
}

void TrackMainWindow::warning( bool error, const QString& msg )
{
	if ( error )
		QMessageBox::critical( this, "Erreur", msg );
	else
		QMessageBox::warning( this, "Alerte", msg );
}

