/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: log.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "log.h"

#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qinputdialog.h>
#include <qmessagebox.h>

Log::Log( QWidget* parent, const char* name )
:	 QDialog( parent, name, false, WDestructiveClose )
{
	resize( 455, 200 );

	setCaption( "LOG" );
	QGridLayout* logLayout = new QGridLayout( this ); 
	logLayout->setSpacing( 6 );
	logLayout->setMargin( 11 );

	multiLineEdit_ = new QMultiLineEdit( this );
	multiLineEdit_->setFocus();
	multiLineEdit_->setReadOnly( true );

	logLayout->addMultiCellWidget( multiLineEdit_, 0, 0, 0, 3 );

	QPushButton* btnSearch = new QPushButton( "&Chercher", this );
	logLayout->addWidget( btnSearch, 1, 0 );
	connect( btnSearch, SIGNAL(clicked()), SLOT(searchFirst()) );

	QPushButton* btnNext = new QPushButton( "&Suivant", this );
	logLayout->addWidget( btnNext, 1, 1 );
	connect( btnNext, SIGNAL(clicked()), SLOT(searchNext()) );

	logLayout->addItem( new QSpacerItem( 20, 20 ), 1, 2 );

	QPushButton* btnClose = new QPushButton( "&Fermer", this );
	logLayout->addWidget( btnClose, 1, 3 );
	connect( btnClose, SIGNAL(clicked()), SLOT(close()) );
}

void Log::clear()
{
	multiLineEdit_->clear();
}

void Log::append( const QString& line )
{
	multiLineEdit_->insertLine( line );
	multiLineEdit_->setCursorPosition( multiLineEdit_->numLines(), 0 );
}

void Log::setAutoUpdate( bool enable )
{
	multiLineEdit_->setAutoUpdate( enable );
	repaint( true );
}

bool Log::autoUpdate()	const
{
	return multiLineEdit_->autoUpdate();
}

void Log::repaint( bool paint )
{
	if ( paint )
		multiLineEdit_->setCursorPosition( multiLineEdit_->numLines(), 0 );
	QDialog::repaint( paint );
}

void Log::searchFirst()
{
	int col;
	multiLineEdit_->getCursorPosition( &beginningAtLine_, &col );
	line_ = beginningAtLine_;
	bool ok;
	searchText_ =
		QInputDialog::getText( "CHERCHER", "text =", searchText_, &ok, this );
	if ( ! ok )
		searchText_ = "";
	else
		search();
}

void Log::searchNext()
{
	if ( ! searchText_.isEmpty() )
	{
		++line_;
		search();
	}
}
	
void Log::search()
{
	int lastLine = multiLineEdit_->numLines();
	while ( true )
	{
		do
		{
			// on chercher la position de searchText_ dans la ligne 'line_'.
			const int p =
						multiLineEdit_->textLine( line_ ).find( searchText_ );
			if ( p != -1 )
			{
				// searchText_ est prsent on le 'sur-ligne'.
				multiLineEdit_->setCursorPosition( line_, p, false );
				multiLineEdit_->setCursorPosition( line_, 
											p + searchText_.length(), true );
				return;
			}
		} while ( ++line_ < lastLine );

		if ( lastLine == multiLineEdit_->numLines() && beginningAtLine_ != 0 )
		{
			const int r =QMessageBox::information( this, "RECHERCHER",
										"Continuer au dbut ?", "Oui", "Non" );
			if ( r == 1 )
				break;
			line_ = 0;
			lastLine = beginningAtLine_;
		}
		else
			break;
	}
}

