/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: tracksimulator.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "tracksimulator.h"
#include "dialogresults.h"
#include "dialogpauseon.h"
#include "log.h"

#include "script/script.h"
#include "script/dialogscript.h"

#include "widgettrack/dimensions.h"
#include "widgettrack/widgettrack.h"
#include "config/config.h"
#include "divers/ext.h"
#include "divers/outputpoint.h"
#include "pixel/outputpixel.h"
#include "icons/icons.h"

#include <qapplication.h>	// qApp

#include <qmenubar.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qcheckbox.h>
#include <qstatusbar.h>
#include <qlayout.h>
#include <qfiledialog.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qfileinfo.h>
#include <qaccel.h>

#include <cmath>	// pour abs

//	****
//	Log : lower/raise
//
//		Est cache si :
//			- Timer passe  zro.
//			- Timer  zro au dmarrage de la simulation.
//			- sur pause off si timer  zro
//
//		Est rendu visible :
//			- Si trace de log et timer ne vaut pas zro.
//			- Si pause on *et* timer vaut zro, si timer ne vaut pas
//			  zro c'est l'utilisateur qui  fait passer la log en
//			  arrire plan.
//			- Si timer passe de zro a positif.
//	****

namespace
{
	enum { NoTimer = -1, TimerPause = -2 };
	const QString	GB_labelBtnPauseOn( "S&ur" );
	const uint		GB_metersPerPixel	= 10;
	const uint		GB_secondesPerTop	= 1;
	const double	GB_minutesPerTop	= GB_secondesPerTop/60.0;
	const double	GB_hoursPerTop		= GB_minutesPerTop/60.0;

	inline double elapsedMinutes( int tops )
	{
		return tops * GB_minutesPerTop;
	}

	inline double elapsedHours( int tops )
	{
		return tops * GB_hoursPerTop;
	}

	inline int pixelsToMeters( int totalPixels )
	{
		return totalPixels * GB_metersPerPixel;
	}

	inline double Kmeters( int distance )
	{
		return distance / 1000.0;
	}

	QDateTime	GB_lastModified;
};

TrackSimulator::TrackSimulator( QWidget* parent, const char* name )
:	 TrackMainWindow( TrackMainWindow::TrackSimulator, parent, name )
	,f1Stream_( &f1File_ )
	,startCoords_(-1)
	,idTimer_(NoTimer)
{
	setIconText( "Simulateur: " + iconText() );
	winResults_ = new DialogResults( this );
	winResults_->move( config_.windowResultsX(), config_.windowResultsY() );

	const Icons& icons = Icons::instance();
	//	---
	//	
	//	---
	btnScript_ = new QPushButton( "&Script", parentTopBar() );
	btnScript_->setToggleButton( true );
	btnScript_->setOn( Script::instance().on() );
	connect( btnScript_, SIGNAL(toggled(bool)), SLOT(script(bool)) );
	addTopBar( btnScript_ );

	btnLoadDriver_ =
		new QPushButton( icons.fileOpen(), "&Pilote", parentTopBar() );
	addTopBar( btnLoadDriver_ );
	connect( btnLoadDriver_, SIGNAL(clicked()), SLOT(loadDriver()) );

	//	---
	//	Bouttons de contrle de la simulations
	QHBoxLayout* ctrlLayout = new QHBoxLayout;

	btnGo_ = new QPushButton( icons.start(), "&Go", parentTopBar() );
	ctrlLayout->addWidget( btnGo_, 1 );
	connect( btnGo_, SIGNAL(clicked()), SLOT(go()) );

	btnPause_ = new QPushButton( icons.pause(), "&Pause", parentTopBar() );
	btnPause_->setToggleButton( true );
	ctrlLayout->addWidget( btnPause_, 1 );
	connect( btnPause_, SIGNAL(toggled(bool)), SLOT(pause(bool)) );

	btnPauseOn_ = new QPushButton( GB_labelBtnPauseOn, parentTopBar() );
	ctrlLayout->addWidget( btnPauseOn_, 1 );
	connect( btnPauseOn_, SIGNAL(clicked()), SLOT(pauseOn()) );

	btnStep_ = new QPushButton( icons.step(), "P&as", parentTopBar() );
	btnStep_->setAutoRepeat( true );
	ctrlLayout->addWidget( btnStep_, 1 );
	connect( btnStep_, SIGNAL(clicked()), SLOT(step()) );
	btnStep_->setDisabled( true );

	btnStop_ = new QPushButton( icons.stop(), "&Stop", parentTopBar() );
	ctrlLayout->addWidget( btnStop_, 1 );
	connect( btnStop_, SIGNAL(clicked()), SLOT(stop()) );
	btnStop_->setDisabled( true );

	addTopBar( ctrlLayout );
	
	addSeparatorTopBar();

	//	---
	//
	//	---
	sliderTimerInterval_ =
			new QSlider( 0, Config::timerMax, Config::timerStep,
						config_.timer(), QSlider::Horizontal, parentTopBar() );
	sliderTimerInterval_->setTickmarks( QSlider::Both );
	connect( sliderTimerInterval_,	SIGNAL(valueChanged(int)),
									SLOT(timerIntervalChanged(int)) );
	QToolTip::add( sliderTimerInterval_,
					" droite temps rel.\n"
					" gauche animation la plus rapide.\n"
					"Chaque graduation reprsente "
						+ QString::number(Config::timerStep) + " ms."
					);

	addTopBar( sliderTimerInterval_ );

	addSeparatorTopBar();

	//	----
	//
	//	----
	lcdSpeed_ = new QLCDNumber( parentTopBar() );
	lcdSpeed_->setSegmentStyle( QLCDNumber::Flat );
	QPalette palLCD = lcdSpeed_->palette();
	palLCD.setColor( QPalette::Active, QColorGroup::Text, Qt::green );
	palLCD.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );
	palLCD.setColor( QPalette::Active, QColorGroup::Base, Qt::black );
	palLCD.setColor( QPalette::Active, QColorGroup::Background, Qt::black );

	palLCD.setColor( QPalette::Inactive, QColorGroup::Text, Qt::green );
	palLCD.setColor( QPalette::Inactive, QColorGroup::Foreground, Qt::green );
	palLCD.setColor( QPalette::Inactive, QColorGroup::Base, Qt::black );
	palLCD.setColor( QPalette::Inactive, QColorGroup::Background, Qt::black );

	lcdSpeed_->setFrameShape( QFrame::StyledPanel );
    lcdSpeed_->setFrameShadow( QFrame::Raised );
	lcdSpeed_->setPalette( palLCD );

	addTopBar( lcdSpeed_ );
	addTopBar( new QLabel( "km/h", parentTopBar() ) );

	lcdDistance_ = new QLCDNumber( parentTopBar() );
	lcdDistance_->setSegmentStyle( QLCDNumber::Flat );
	lcdDistance_->setFrameShape( QFrame::StyledPanel );
    lcdDistance_->setFrameShadow( QFrame::Raised );
	addTopBar(lcdDistance_  );
	addTopBar( new QLabel( "m", parentTopBar() ) );

	addSeparatorTopBar();

	QToolButton* btnQuit = new QToolButton( parentTopBar() );
	btnQuit->setIconSet( icons.exit() );
	addTopBar( btnQuit );
	connect( btnQuit, SIGNAL(clicked()), SLOT(close()) );
	//	----

	connect( widgetTrack_, SIGNAL(mouseLeftReleasePixel(QMouseEvent*,Pixel&)),
							SLOT(mouseRightClicked(QMouseEvent*,Pixel&)) );
}

void TrackSimulator::mouseRightClicked( QMouseEvent* me, Pixel& pixel )
{
	QString msg = outPutPixel( pixel );
	if ( me->state() & ControlButton )
	{
		pauseOn_.x = pixel.x();
		pauseOn_.y = pixel.y();
		btnPauseOn_->setText( GB_labelBtnPauseOn + ' ' + 
									outPutPoint( pauseOn_.x, pauseOn_.y ) );
		msg = "Pause sur le pixel " + msg;
	}
	statusBar_.message( msg );
}

void TrackSimulator::pauseOn()
{
	DialogPauseOn dpo( pauseOn_.x, pauseOn_.y, this );
	const QPoint topLeft( btnPauseOn_->x(),
									btnPauseOn_->y()+ btnPauseOn_->height() );
	dpo.move( mapToParent( topLeft ) );

	if ( dpo.exec() )
	{
		if ( dpo.pauseX() == -1 )
		{
			pauseOn_.x = -2;
			btnPauseOn_->setText( GB_labelBtnPauseOn + " traces" );
		}
		else
		{
			pauseOn_.x = dpo.pauseX();
			pauseOn_.y = dpo.pauseY();
			btnPauseOn_->setText( GB_labelBtnPauseOn + ' ' +
								outPutPoint( dpo.pauseX(), dpo.pauseY() ) );
		}
	}
	else
	{
		pauseOn_.x = -1;
		btnPauseOn_->setText( GB_labelBtnPauseOn );
	}
}

void TrackSimulator::timerIntervalChanged( int value )
{
	if ( idTimer_ != NoTimer && idTimer_ != TimerPause )
	{
		killTimer( idTimer_ );
		idTimer_ = startTimer( value );
		if ( logDebug_ )
		{
			logDebug_->setAutoUpdate( value != 0 || btnPause_->isOn() );
			if ( value == 0 )
				logDebug_->lower();
			else
				logDebug_->raise();
		}
	}
}

void TrackSimulator::initResults()
{
	winResults_->setTime( elapsedMinutes(topTimer_) );
	winResults_->setDistance( static_cast<int>(totalDistanceMeters_) );
	winResults_->setSpeedMax( maxSpeed_ );
	winResults_->setSpeedMean( Kmeters(totalDistanceMeters_) / elapsedHours(topTimer_) );
}

void TrackSimulator::appendLogDebug( const QString& line )
{
	if ( ! logDebug_ )
	{
		logDebug_ = new Log( this );
		QAccel* accel = new QAccel( logDebug_ );
		accel->connectItem( accel->insertItem(CTRL+Key_F1),
													this, SIGNAL(helpLog()) );
		logDebug_->move( config_.windoLogX(), config_.windoLogY() );
		logDebug_->resize( config_.windoLogW(), config_.windoLogH() );
		logDebug_->show();
		if ( sliderTimerInterval_->value() == 0 && ! btnPause_->isOn() )
			logDebug_->lower();
		logDebug_->setAutoUpdate(
					sliderTimerInterval_->value() != 0 || btnPause_->isOn() );
	}
	logDebug_->append( line );
}

void TrackSimulator::printErrorScript()
{
	QFile logS( Script::logFile );
	if ( logS.open( IO_ReadOnly ) )
	{
		appendLogDebug( "" );
		appendLogDebug( "Canal d'erreur du script" );
		QTextStream s( &logS );
		appendLogDebug( s.read() );
	}
}

bool TrackSimulator::execScript()
{
	Script& execScript = Script::instance();
	if ( execScript.on() )
	{
		statusBar_.message( "Excution du script ..." );
		qApp->processEvents();
		if ( ! execScript.exec() )
		{
			printErrorScript();
			statusBar_.message( "chec excution du script." );
			return false;
		}
	}

	return true;
}

void TrackSimulator::script( bool on )
{
	Script& execScript = Script::instance();
	execScript.setOn( on );
	if ( on )
	{
		DialogScript ds( &execScript, this, 0 );
		const QPoint topLeft( btnScript_->x(),
									btnScript_->y()+ btnScript_->height() );
		ds.move( mapToParent( topLeft ) );

		if ( ds.exec() == 0 )
			execScript.setOn( false );

		btnScript_->blockSignals( true );
		btnScript_->setOn( execScript.on() );
		btnScript_->blockSignals( false );
	}
}

void TrackSimulator::pause( bool pauseOn )
{
	btnStep_->setDisabled( ! pauseOn );
	switch ( pauseOn )
	{
	case true:
		if ( idTimer_ != NoTimer )
		{
			killTimer( idTimer_ );
			idTimer_ = TimerPause;
		}

		if ( logDebug_ && ! logDebug_->autoUpdate() )
		{
			logDebug_->setAutoUpdate( true );
			if ( sliderTimerInterval_->value() == 0 )
				logDebug_->raise();
		}

		statusBar_.message( "Pause...", 4000 );
		break;

	case false:
		// Le boutton Pause peut tre 'click' mme si aucune piste ou
		// aucun pilote n'est charg.
		if ( idTimer_ == TimerPause )
		{
			assert( startCoords_ != -1 );
			idTimer_ = startTimer( sliderTimerInterval_->value() );
			if ( logDebug_ )
			{
				if ( sliderTimerInterval_->value() == 0 )
					logDebug_->lower();
				logDebug_->setAutoUpdate( sliderTimerInterval_->value() != 0 );
			}
		}
	}
}

void TrackSimulator::step()
{
	if ( startCoords_ != -1 && idTimer_ == TimerPause )
		timerEvent( NULL );
}

void TrackSimulator::stop()
{
	if ( idTimer_ != TimerPause )
	{
		assert( idTimer_ != NoTimer );
		killTimer( idTimer_ );
	}

	idTimer_ = NoTimer;
	f1Stream_.read();	// on vide.

	lcdSpeed_->display( 0 );

	btnLoadDriver_->setDisabled( false );
	btnGo_->setDisabled( false );
	btnScript_->setDisabled( false );
	btnStop_->setDisabled( true );
	btnStep_->setDisabled( true );

	if ( btnPause_->isOn() && pauseOn_.x != -1 )
		btnPause_->setOn( false );

	if ( logDebug_ )
	{
		if ( ! logDebug_->autoUpdate() )
			logDebug_->setAutoUpdate( true );

		if ( sliderTimerInterval_->value() == 0 )
			logDebug_->raise();
	}

	Script& execScript = Script::instance();
	if ( execScript.on() )
	{
		printErrorScript();
		statusBar_.message(	execScript.error() != "" 
						?	execScript.error()
						:	QString( "Le script c'est termin normallement." )
						);
	}
	else
		statusBar_.message( "Finie." );
}

bool TrackSimulator::loadTrack()
{
	if ( widgetTrack_->load( trackName_, false ) )
	{
		QString msg;
		
		int count = widgetTrack_->countColor( config_.driverColor() );
		if ( count != 0 )
			msg = "La couleur du pilote est dj utilise pour la piste.";

		count = widgetTrack_->countColor( config_.traceColor() );
		if ( count != 0 )
			msg += "\nLa couleur de 'trace' est dj utilise pour la piste.";

		if ( ! msg.isEmpty() )
		{
			QMessageBox::critical( this, "CONFIG INVALIDE",
				msg + "\n\nCorriger la configuration avant de continuer !" );
		}
		else
			return true;
	}

	return false;
}

bool TrackSimulator::loadFileDriver()
{
	if ( f1File_.open( IO_ReadOnly ) )
	{
		//	Ligne 1 : Nom du pilote.
		winResults_->setDriver( f1Stream_.readLine() );
		winResults_->show();

		//	Ligne 2 : Nom du fichier de description de la piste.
		const QString piste = f1Stream_.readLine();
		trackName_	=	! QFile::exists( piste )
					?	config_.pathTracks() + '/' + piste
					:	piste
					;
		setCaption( trackName_ ); 

		startCoords_ = f1File_.at();
		return loadTrack();
	}

	return false;
}

bool TrackSimulator::loadDriver()
{
	f1File_.close();
	const QString file =
			QFileDialog::getOpenFileName( config_.pathDrivers(),
								"*."+ext::result, NULL, 0, "Charger Pilote" );

	if ( ! file.isEmpty() )
	{
		if ( ! f1File_.name().isEmpty() )	// change de fichier f1 donc on
			btnScript_->setOn( false );		// annule l'excution du script.

		f1File_.setName( file );
		GB_lastModified = QFileInfo( file ).lastModified();
		return loadFileDriver();
	}

	return false;
}

void TrackSimulator::start()
{
	assert( startCoords_ != -1 );

	if ( idTimer_ != NoTimer && ! btnPause_->isOn() )
	{
		killTimer( idTimer_ );
		idTimer_ = NoTimer;
	}

	if ( ! loadTrack() )
		return;

	f1File_.at( startCoords_ );
	char sep;
	f1Stream_ >> prevX_ >> sep >> prevY_;
	if ( prevX_ != 40 && prevY_ != 4 )
	{
		QMessageBox::critical( this, "FICHIER INVALIDE",
				"La position de dpart doit tre en [40,4],\n"
				"elle est  la position " + outPutPoint(prevX_,prevY_) );
		return;
	}

	if ( btnPause_->isOn() )
	{
		idTimer_ = TimerPause;
		btnStep_->setDisabled( false );
	}

	btnLoadDriver_->setDisabled( true );
	btnGo_->setDisabled( true );
	btnScript_->setDisabled( true );
	btnStop_->setDisabled( false );

	widgetTrack_->paintPixel( config_.driverColor(),
											Dim::numPixel( prevX_, prevY_ ) );

	topTimer_ = totalDistanceMeters_ = 0;
	maxSpeed_ = 0;
	winResults_->setCaption( "EN COURS..." );
	winResults_->setInfo( "EN COURS..." );
	initResults();
	if ( idTimer_ != TimerPause )
	{
		idTimer_ = startTimer( sliderTimerInterval_->value() );
		statusBar_.message( "Running..." );
	}
	else
		statusBar_.message( "Ready..." );
}

void TrackSimulator::go()
{
	if ( logDebug_ )
	{
		logDebug_->clear();
		if ( sliderTimerInterval_->value() == 0  && ! btnPause_->isOn() )
			logDebug_->lower();
		logDebug_->setAutoUpdate(
					sliderTimerInterval_->value() != 0 || btnPause_->isOn() );
	}

	if ( startCoords_ == -1 && ! loadDriver() )
		return;

	if ( execScript() )
	{
		const QFileInfo fi( f1File_ );
		if ( fi.lastModified() != GB_lastModified )
		{
			f1File_.close();
			GB_lastModified = fi.lastModified();
			if ( ! loadFileDriver() )
				return;
			statusBar_.message( "Le fichier " + f1File_.name()
														+ " a t recharg." );
		}

		start();
	}
}

void TrackSimulator::timerEvent( QTimerEvent* )
{
	assert( startCoords_ != -1 );

	if ( f1Stream_.atEnd() )
	{
		stop();
		winResults_->setCaption( "VAINQUEUR" );
		winResults_->setInfo( "Bravo" );
		initResults();
		if ( ! winResults_->isVisible() )
			winResults_->show();
		else
			winResults_->raise();
	}
	else
	{
		int x, y;
		do
		{
			f1Stream_ >> x;
			switch ( x )
			{
			case -1:
				appendLogDebug( f1Stream_.readLine() );
				if ( pauseOn_.x == -2 )
					btnPause_->setOn( true );
				break;

			case -2:
				{
					int nPixel;
					f1Stream_ >> nPixel;
					if ( Dim::numPixelValid( nPixel ) )
						widgetTrack_->paintPixel(config_.traceColor(), nPixel);
					else
					{
						btnPause_->setOn( true );
						const QString msg = "***> TRACE: pixel "
								+ QString::number(nPixel) + " invalide !";
						appendLogDebug( msg );
						statusBar_.message( msg );
						return;
					}
				}
				break;

			default:
				{
					char sep;
					f1Stream_ >> sep >> y;
				}
			}
		} while ( x == -1 || x == -2 );

		if ( x != 0 && y != 0 )	// FIXME: comprends pas !
		{
			++topTimer_;
			const int pixelP	= Dim::numPixel(prevX_,prevY_);
			const int pixel		= Dim::numPixel(x,y);
			const int pixelErr	= widgetTrack_->moveCar( 
							config_.driverColor(), config_.traceColor(),
															pixelP, pixel );
			if ( pixelErr != -1 )
			{
				stop();

				const int crossWidth = config_.crossSize();
				
				int fxPixel = pixelErr - crossWidth / 2;
				int lxPixel = pixelErr + crossWidth / 2;
				
				do widgetTrack_->paintPixel( config_.crossColor(), fxPixel );
				while ( ++fxPixel <= lxPixel );

				fxPixel = pixelErr - (Dim::maxX*crossWidth);
				lxPixel = pixelErr + (Dim::maxX*crossWidth);
				do 
				{
					widgetTrack_->paintPixel( config_.crossColor(), fxPixel );
					fxPixel += Dim::maxX;
				} while ( fxPixel <= lxPixel );

				winResults_->setCaption( "SORTIE DE PISTE" );
				QString msg;
				QTextOStream( &msg )
					<< "Collision au pixel " << pixelErr
					<< "\nTrajectoire " 
					<< pixelP << ' ' << outPutPoint(prevX_,prevY_)
					<< " -> " << pixel << ' ' << outPutPoint( x, y );

				winResults_->setInfo( msg );
				initResults();
				
				if ( ! winResults_->isVisible() )
					winResults_->show();
				else
					winResults_->raise();

				return;
			}
							
			const uint distanceX	= std::abs(x - prevX_);
			const uint distanceY	= std::abs(y - prevY_);
			const uint meters		= pixelsToMeters( distanceX + distanceY );

			totalDistanceMeters_ += meters;
			const double speedKH = Kmeters(meters) / (GB_hoursPerTop);
			lcdSpeed_->display( speedKH );
			lcdDistance_->display( static_cast<int>(totalDistanceMeters_) );
			if ( speedKH > maxSpeed_ )
				maxSpeed_ = speedKH;

			if ( winResults_->isVisible() )
				initResults();

			if ( btnPause_->isOn() )
			{
				QString m;
				QTextOStream( &m )
					<< "Trajectoire " 
						<< Dim::numPixel(prevX_,prevY_) << ' '
						<< outPutPoint(prevX_, prevY_)
					<< " -> "
						<< pixel << ' '
						<< outPutPoint(x, y)
					<< ", distance (x,y) = " << outPutPoint(distanceX,distanceY)
					<< " = " << meters << 'm'
					;
				statusBar_.message(m);
			}

			prevX_ = x;
			prevY_ = y;
			if ( x == pauseOn_.x && y == pauseOn_.y )
			{
				statusBar_.message(
						"Bascule sur pause au pixel " + outPutPoint( x, y ) );
				btnPause_->setOn( true );
			}
		}
	}
}

TrackSimulator::~TrackSimulator()
{
	config_.setTrackSimulatorXY( x(), y() );
	if ( winResults_ )
		config_.setWindowResultsXY( winResults_->x(), winResults_->y() );
	if ( logDebug_ )
		config_.setWindowLog( logDebug_->x(), logDebug_->y(),
									logDebug_->width(), logDebug_->height() );
}

