/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: tracksimulator.h
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#ifndef TRACKSIMULATOR_H
#define TRACKSIMULATOR_H

//============================================================================
//	TrackSimulator:
//		Joue les rsultats d'un pilote sur une piste.
//============================================================================

#include <trackmainwindow/trackmainwindow.h>

#include <qfile.h>
#include <qtextstream.h>
#include <qguardedptr.h> 

class Log;
class Pixel;
class QSlider;
class QLCDNumber;
class QPushButton;
class DialogResults;
class TrackSimulator : public TrackMainWindow
{
Q_OBJECT
public:
	TrackSimulator( QWidget* parent, const char* name );

	using	TrackMainWindow::trackName;

	~TrackSimulator();

signals:
	void helpLog();

private slots:
	void			script(bool);	// Active/dsactive l'excution d'un
									// script avant la lecture d'un fichier .f1

	bool			loadDriver();	// demande un fichier .f1

	void			go();			// lance ou re-lance une simulation,
									// demande le fichier .f1 par loadDriver
									// si aucun fichier n'a t slectionn.
									// Appel de execScrip, recharge le fichier
									// .f1 si il a t modifi.

	void			timerIntervalChanged( int );

	void			pause( bool );	// suspend ou redmarre la simulation.

	void			pauseOn();		// Permet de dfinir un point d'arrt
									// Si la F1 passe sur le pixel slectionn
									// la simulation bascule sur pause.

	void			step();			// excute le fichier .f1 ligne par ligne
									// si on est en pause.

	void			stop();			// Arrte la simulation.

	void			mouseRightClicked( QMouseEvent*, Pixel& );
									// Affiche les coordonnes du pixel
									// 'click' dans la status barre, si
									// CTRL press le pixel servira de point
									// d'arrt si la F1 passe dessus.

private:
	void			printErrorScript();
	bool			execScript();		// Excute le script si actif.

	bool			loadTrack();		// charge la piste et test la validit
										// de la config.

	bool			loadFileDriver();	// charge le fichier .f1

	void			start();			// dmarre la simulation.
										// charge la piste.
	
	override void	timerEvent( QTimerEvent* );

	void			initResults();

	void			appendLogDebug( const QString& line );

private:
	QFile			f1File_;
	QTextStream		f1Stream_;
	int				startCoords_;

	int				idTimer_;
	QSlider*		sliderTimerInterval_;

	QLCDNumber*		lcdSpeed_;
	QLCDNumber*		lcdDistance_;

	DialogResults*	winResults_;
	uint			totalDistanceMeters_;
	uint			topTimer_;
	double			maxSpeed_;
	int				prevX_, prevY_;

	struct	{ int x, y; } pauseOn_;

	QPushButton*		btnScript_;
	QPushButton*		btnLoadDriver_;
	QPushButton*		btnGo_;
	QPushButton*		btnPause_;
	QPushButton*		btnPauseOn_;
	QPushButton*		btnStop_;
	QPushButton*		btnStep_;

	QGuardedPtr<Log>	logDebug_;
};

#endif	//TRACKSIMULATOR_H


