/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: dialogpalettecolor.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "dialogpalettecolor.h"

#include "pixel/listshapepixel.h"

#include <qbuttongroup.h>
#include <qlayout.h>
#include <qradiobutton.h>

DialogPaletteColor::DialogPaletteColor( QWidget* parent, const char* name, bool modal )
:	 QDialog( parent, name, modal )
{
	setCaption( "Couleurs" );

	gridLayout_ = new QGridLayout( this ); 
    gridLayout_->setSpacing( 2 );
    gridLayout_->setMargin( 2 );
	
	buttonGroup_ = new QButtonGroup( this  );
	buttonGroup_->setFrameStyle( QFrame::Panel | QFrame::Raised );
	connect( buttonGroup_, SIGNAL(clicked(int)), SLOT(onClick(int)) );
	gridLayout_->addWidget( buttonGroup_, 0, 0 );

	buttonGroup_->setColumnLayout( 0, Qt::Vertical );
	buttonGroup_->layout()->setSpacing( 0 );
	buttonGroup_->layout()->setMargin( 0 );
	QGridLayout* buttonGroup_Layout = new QGridLayout( buttonGroup_->layout() );
	buttonGroup_Layout->setAlignment( Qt::AlignTop );
	buttonGroup_Layout->setSpacing( 6 );
	buttonGroup_Layout->setMargin( 11 );

	ListShapePixel& lsp = ListShapePixel::instance();
	QPalette p = palette();
	for ( int i = 0; i < ListShapePixel::max; ++i )
	{
		const ShapePixel& shape = lsp[i];
		QRadioButton* rb = new QRadioButton( shape.name(), buttonGroup_ );
		p.setColor( QPalette::Active, QColorGroup::Foreground, shape.color() );
		p.setColor( QPalette::Inactive, QColorGroup::Foreground, shape.color() );
		rb->setPalette( p );
		buttonGroup_Layout->addWidget( rb, i, 0 );
	}
	buttonGroup_->setButton( 0 );
}

void DialogPaletteColor::polish()
{
	QDialog::polish();
	setFixedSize( sizeHint() );
}

void DialogPaletteColor::setColor( const QColor& c )
{
	buttonGroup_->setButton( ListShapePixel::id(c) );
	emit colorChanged( c );
}

void DialogPaletteColor::onClick( int index )
{
	emit colorChanged( ListShapePixel::instance()[index].color() );
}

