/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: editselectfilewidget.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "editselectfilewidget.h"

#include <qlineedit.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qfile.h>
#include <qfiledialog.h>

EditSelectFileWidget::EditSelectFileWidget( Type type, QWidget* parent,
															const char* name )
:	 QWidget( parent, name )
	,type_(type)
	,valid_(true)
	,filters_("*")
{
	setMinimumWidth( 340 );

	QHBoxLayout* hLayout = new QHBoxLayout( this );

	lineEdit_ = new QLineEdit( this );
	connect( lineEdit_, SIGNAL(textChanged(const QString&)),
										SLOT(fileChanged(const QString&)) );
	hLayout->addWidget( lineEdit_, 1 /*stretch factor*/ );

	QPushButton* openD = new QPushButton( "&Parcourir...", this );
	connect( openD, SIGNAL(clicked()), SLOT(open()) );
	hLayout->addWidget( openD );

	original_ = lineEdit_->palette();
}

void	EditSelectFileWidget::setFile( const QString& file )
{
	lineEdit_->setText( file );
}

QString	EditSelectFileWidget::file() const
{
	return lineEdit_->text(); 
}

void EditSelectFileWidget::fileChanged( const QString& file )
{
	//FIXME: comprends pas: marche pas en SGI.
	const bool changedValid = QFile::exists( file );
	if ( changedValid != valid_ )
	{
		QPalette p = original_;
		if ( ! changedValid )
			p.setColor( QColorGroup::Text, Qt::red );
		lineEdit_->setPalette( p );
		valid_ = changedValid;
	}
}

void EditSelectFileWidget::open()
{
	QString file = type_ == Path
				 ? QFileDialog::getExistingDirectory( lineEdit_->text(), this,
													 0, QString::null, true )
				 : QFileDialog::getOpenFileName( lineEdit_->text(), filters_,
								 									this, 0 )
				 ;

	if ( ! file.isEmpty() )
	{
		if ( file[file.length()-1] == '/' )
			file.truncate( file.length() - 1 );
		lineEdit_->setText( file );
	}
}

