/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: widgetinputpixel.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "widgetinputpixel.h"

#include "widgettrack/dimensions.h"

#include <qlayout.h>
#include <qspinbox.h>
#include <qlcdnumber.h>
#include <qlabel.h>

WidgetInputPixel::WidgetInputPixel( const QString& title, QWidget* parent,
															const char* name )
:	 QGroupBox( title, parent, name )
{
	setColumnLayout( 0, Qt::Vertical );
	layout()->setSpacing( 0 );
	layout()->setMargin( 0 );

	QGridLayout* groupBoxLayout = new QGridLayout( this->layout() );
	groupBoxLayout->setAlignment( Qt::AlignTop );
	groupBoxLayout->setSpacing( 6 );
	groupBoxLayout->setMargin( 11 );

	QLabel* labelX = new QLabel( "X =", this );
	groupBoxLayout->addWidget( labelX, 0, 0 );
	spinBoxX_ = new QSpinBox( 0, Dim::maxX-1, 25, this );
	groupBoxLayout->addWidget( spinBoxX_, 0, 1 );
	connect( spinBoxX_, SIGNAL(valueChanged(int)), SLOT(xyChanged(int)) );

	QLabel* labelY = new QLabel( "Y =", this );
	groupBoxLayout->addWidget( labelY, 0, 2 );
	spinBoxY_ = new QSpinBox( 0, Dim::maxY-1, 10, this );
	groupBoxLayout->addWidget( spinBoxY_, 0, 3 );
	connect( spinBoxY_, SIGNAL(valueChanged(int)), SLOT(xyChanged(int)) );

	groupBoxLayout->addWidget( new QLabel( "Pixel n", this ), 0, 4 );
	LCDPixel_ = new QLCDNumber( this );
	LCDPixel_->setSegmentStyle( QLCDNumber::Flat );
	groupBoxLayout->addWidget( LCDPixel_, 0, 5 );

	connect( spinBoxX_, SIGNAL(valueChanged(int)),
												SIGNAL(valueChanged(int)) );
	connect( spinBoxY_, SIGNAL(valueChanged(int)),
												SIGNAL(valueChanged(int)) );
}

void	WidgetInputPixel::setXY( int x, int y )
{
	spinBoxX_->setValue( x );
	spinBoxY_->setValue( y );
}

int		WidgetInputPixel::pixelX()	const { return spinBoxX_->text().toInt(); }
int		WidgetInputPixel::pixelY()	const { return spinBoxY_->text().toInt(); }

int		WidgetInputPixel::pixel()	const
{
	return Dim::numPixel( pixelX(), pixelY() ); 
}

void	WidgetInputPixel::xyChanged( int )
{
	LCDPixel_->display( pixel() );
}

